{{ $layout := "col-lg-12" }}
{{- partial "header_page.html" (dict "currentNode" . "layout" $layout) }}

<div class="container">
  <div class="row">
    <div class="col-md-8">
      {{ partial "next-prev-page.html" . }}
      {{ block "main_body" . }}
      {{ end }}
    </div>
    <div class="col-md-4">
      {{- partial "sidebar.html" . }}
    </div>
  </div>
</div>

{{- partial "footer.html" . }}
