{{ $layout := "col-lg-8 offset-lg-2" }}

{{- partial "header_page.html" (dict "currentNode" . "layout" $layout) }}

<div class="container">
  <div class="row">
    <div class="{{ $layout }}">
      <br>
      <div class="card bg-light my-5 mx-auto text-center" style="max-width: 28rem;">
        <div class="card-header">404.html</div>
        <div class="card-body">
          <h1 class="card-title">Page not found</h1>
          <p class="card-text my-3"><h6>Looks like the page you are going to visit doesn't exist. Please check the URL and try your luck again.</h6></p>
          <a href="{{ "/" | relURL }}" class="btn btn-secondary my-3">Take me Home</a>
        </div>
      </div>
    </div>
  </div>
</div>

{{- partial "footer.html" . }}

