<!--
Highlights file and provides download link.
Argument 1 = File path below /src/
-->
{{- $src := (.Get "file") }}
{{- $lang := .Get "language" | default ("python") }}
{{- $opt := .Get "opt" | default ("linenos=table") }}
{{- $path := (printf "/static/py/auto/Examples/%s" $src) }}
{{- $file := $path | readFile }}
{{- highlight (print $file) $lang $opt }}
<a class="linktofile"
   href="{{.Site.Params.url_blob}}/auto/Examples/{{(printf $src)}}"/>auto/Examples/{{$src}}</a>
