<div class="alert {{ if .IsNamedParams }}
{{with .Get "theme" }}alert-{{.}}{{else}}alert-info{{end}}"
{{else}}
	{{with .Get 0 }}alert-{{.}}{{else}}alert-info{{end}}"
{{end}} 
role="alert">{{ .Inner | markdownify }}</div>
