//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/SpheroidalSegment.h
//! @brief     Defines class SpheroidalSegment.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_SPHEROIDALSEGMENT_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_SPHEROIDALSEGMENT_H

#include "Sample/Particle/HardFormfactor.h"

//! A truncated spheroid.
//! An ellipsoid with two equal axis, truncated by a plane perpendicular to the third axis.

class SpheroidalSegment : public HardFormfactor {
public:
    SpheroidalSegment(double radius_xy, double radius_z, double rm_top, double rm_bottom);

    complex_t formfactorAtBottom(C3 q) const override;

    std::string className() const final { return "SpheroidalSegment"; }

#ifndef SWIG
    SpheroidalSegment* clone() const override
    {
        return new SpheroidalSegment(m_radius_xy, m_radius_z, m_rm_top, m_rm_bottom);
    }
    void validate() const override;

    double radiusXY() const { return m_radius_xy; }
    double radiusZ() const { return m_radius_z; }
    double cutFromTop() const { return m_rm_top; }
    double cutFromBottom() const { return m_rm_bottom; }

    double radialExtension() const override { return m_radius_xy; }
    bool contains(const R3& position) const override;

private:
    double m_radius_xy;
    double m_radius_z;
    double m_rm_top;    //!< height of removed top cap
    double m_rm_bottom; //!< height of removed bottom cap
#endif                  // SWIG
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_SPHEROIDALSEGMENT_H
