//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/Dilute3D.h
//! @brief     Defines class Dilute3D.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2026
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_AGGREGATE_DILUTE3D_H
#define BORNAGAIN_SAMPLE_AGGREGATE_DILUTE3D_H

#include "Sample/Aggregate/Disordered3D.h"

//! Disordered 3D particle structure with dilute approximation (no inter-particle correlations).

class Dilute3D : public Disordered3D {
public:
    Dilute3D(double volume_density, const IParticle& particle);
    ~Dilute3D() override = default;

    std::string className() const final { return "Dilute3D"; }

#ifndef SWIG
    Dilute3D* clone() const override;

    complex_t structureCrossFactor(const WavevectorInfo& q_m,
                                   const WavevectorInfo& q_n) const override;
    complex_t selfPositionFactor(const WavevectorInfo& q_m,
                                 const WavevectorInfo& q_n) const override;
#endif
};

#endif // BORNAGAIN_SAMPLE_AGGREGATE_DILUTE3D_H
