<!DOCTYPE html>
<html lang="{{ .Site.LanguageCode }}">

<head>
<meta charset="utf-8">
<meta http-equiv="X-UA-Compatible" content="IE=edge">
<meta name="viewport" content="width=device-width, initial-scale=1">
<meta name="description" content="">
<meta name="author" content="">
<title>BornAgain-{{ .Site.Params.version_name }} {{ .Title }}</title>
<link rel="icon" href="/img/bornagain-logo.png">

<link href="/css/bootstrap.css" rel="stylesheet">
<link href="/css/theme.css" rel="stylesheet">
<link href="/css/vs.css" rel="stylesheet"> <!-- (highlight) -->
<link href="/css/baguetteBox.css" rel="stylesheet">
<link href="/css/gallery-grid.css" rel="stylesheet">

<!-- Custom Fonts -->
<link href="//maxcdn.bootstrapcdn.com/font-awesome/4.1.0/css/font-awesome.min.css" rel="stylesheet" type="text/css">
<link href='//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,800italic,400,300,600,700,800' rel='stylesheet' type='text/css'>
<link href='https://fonts.googleapis.com/css?family=Roboto' rel='stylesheet' type='text/css'>
<link href="https://fonts.googleapis.com/css?family=Libre+Baskerville" rel='stylesheet'  type='text/css'>

<script src="/js/jquery.js"></script>
<script src="https://bornagainproject.org/js/version_switch.js"></script>

<script>
MathJax = {
  tex: {
    inlineMath: [['$', '$'], ['\\(', '\\)']]
  },
  svg: {
    fontCache: 'global'
  }
};
</script>
<script type="text/javascript" id="MathJax-script" async
  src="https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-svg.js">
</script>

</head>
