//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Base/Type/Span.h
//! @brief     Defines class Span.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_BASE_TYPE_SPAN_H
#define BORNAGAIN_BASE_TYPE_SPAN_H

#include <limits>
#include <utility>

//! An interval. Limits are of type double, and may be infinite.
//! Used for the z-coordinate, especially when slicing form factors.

class Span {
public:
    Span(); //!< Empty span
    Span(double low, double hig);

#ifndef SWIG
    static constexpr double inf = std::numeric_limits<double>::infinity();

    Span operator+(double increment) const;
    Span operator-(double decrement) const { return operator+(-decrement); }

    double low() const;
    double hig() const;
    double center() const;
    double extension() const;

    bool contains(double z) const { return m_low <= z && z <= m_hig; }
    bool empty() const;

    std::pair<double, double> pair() const { return {m_low, m_hig}; }

    //! Returns the union of two Span|s (the minimal interval that contains both input intervals).
    static Span unite(const Span& left, const Span& right);

    //! Returns the intersection of two Span|s
    //! (the maximal interval contained in both input intervals).
    //! If the intersection contains one single number, then an empty Span is returned.
    static Span intersect(const Span& left, const Span& right);

private:
    // members cannot be made const lest we loose the assignement operator:
    double m_low;
    double m_hig;
#endif // SWIG
};

#endif // BORNAGAIN_BASE_TYPE_SPAN_H
