//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Particle/HardFormfactor.h
//! @brief     Defines class HardFormfactor.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2026
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_PARTICLE_HARDFORMFACTOR_H
#define BORNAGAIN_SAMPLE_PARTICLE_HARDFORMFACTOR_H

#include "Sample/Particle/Formfactor.h"

//! Abstract base class for hard particle form factors with well-defined boundaries.

class HardFormfactor : public Formfactor {
#ifndef SWIG
public:
    HardFormfactor* clone() const override = 0;

    //! Checks whether the point of space lies inside the geometric figure.
    virtual bool contains(const R3& position) const = 0;
#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_PARTICLE_HARDFORMFACTOR_H
