//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Flux/ScalarFlux.cpp
//! @brief     Implements class ScalarFlux.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Flux/ScalarFlux.h"

ScalarFlux::ScalarFlux(complex_t kz, Spinor TR)
    : m_kz(kz)
    , m_TR(TR)
{
}

complex_t ScalarFlux::getScalarT() const
{
    return m_TR.u;
}
complex_t ScalarFlux::getScalarR() const
{
    return m_TR.v;
}
