#!/usr/bin/env python3
"""
Spheroid particles attached to the bottom of a vacuum layer.
Uses deposit2D on the vacuum layer with centered alignment.

This example produces the same result as AttachAboveSubstrate.py because
both place particles at the vacuum-substrate interface with their centers
at the interface.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2


def get_sample():
    """
    Two-layer sample (vacuum + substrate) with spheroid particles
    deposited at the bottom of the vacuum layer with centered alignment.
    """
    # Materials
    vacuum = ba.Vacuum()
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 3e-5, 2e-8)

    # Spheroid particle (equatorial radius 4nm, polar radius 6nm)
    ff = ba.Spheroid(4*nm, 6*nm)
    particle = ba.Particle(particle_mat, ff)

    # Shift up so center lands at the interface
    # (deposit2D anchors the bottom by default)
    particle.translate(0, 0, -particle.height_below_center())

    # Layers
    vacuum_layer = ba.Layer(vacuum)
    vacuum_layer.deposit2D(ba.Dilute2D(0.01/nm2, particle))
    substrate_layer = ba.Layer(substrate_mat)

    # Sample
    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 3*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    plotargs = bp.parse_commandline()
    bp.plot_datafield(result, unit_aspect=1, **plotargs)
    bp.export(**plotargs)
