#!/usr/bin/env python3
"""
Conical mesocrystal on a substrate, with material averaging (slicing).
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2, R3


def get_sample():
    # Materials (10x stronger absorption than standard Mesocrystal example)
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-05, 2e-07)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-06, 2e-07)
    vacuum = ba.Vacuum()

    # Basis particle: smaller spheres
    inner_ff = ba.Sphere(2.3*nm)
    inner_particle = ba.Particle(particle_mat, inner_ff)

    # 3D lattice: smaller lattice constant
    lattice = ba.Lattice3D(R3(5*nm, 0, 0), R3(0, 5*nm, 0),
                           R3(0, 0, 5*nm))

    # Crystal
    crystal = ba.Crystal(inner_particle, lattice)

    # Mesocrystal: cone outer shape with alpha=75deg
    outer_ff = ba.Cone(20*nm, 50*nm, 75*deg)
    outer_particle = ba.Mesocrystal(crystal, outer_ff)

    # Layers with slicing for material averaging
    layer_1 = ba.Layer(vacuum)
    layer_1.setNumberOfSlices(5)
    layer_1.deposit2D(ba.Dilute2D(0.0001/nm2, outer_particle))
    layer_2 = ba.Layer(substrate_mat)

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)

    return sample


def get_simulation(sample):
    # Reduced grazing angle for evanescent wave in averaged layer
    # Critical angle ~0.094deg for effective averaged mesocrystal layer
    beam = ba.Beam(1e9, 0.1*nm, 0.08*deg)
    n = 11
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    from bornagain import ba_check
    ba_check.persistence_test(result)
