#!/usr/bin/env python3
"""
GISAS of a sliced composition crossing the vacuum/substrate interface.

Two spherical cups (SphericalSegment, radius 10 nm) of different materials
form a composition that straddles the vacuum/Substrate2 interface:
- Top cup: Ag, height 4 nm
- Bottom cup: Teflon, height 16 nm (2*R - 4), flipped 180 deg around X
The composition is shifted down by 4 nm so it crosses the interface.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    vacuum = ba.Vacuum()
    ag_color = (0.86, 0.24, 0.18)
    ag_mat = ba.RefractiveMaterial("Ag", ag_color, 1.245e-5, 5.419e-7)
    teflon_color = (0.93, 0.72, 0.25)
    teflon_mat = ba.RefractiveMaterial("Teflon", teflon_color, 2.900e-6, 6.019e-9)
    substrate_color = (0.24, 0.50, 0.74)
    substrate_mat = ba.RefractiveMaterial(
        "Substrate2", substrate_color, 3.212e-6, 3.244e-8)

    sphere_radius = 10*nm
    bottom_cup_height = 4*nm

    top_cup = ba.Particle(
        ag_mat,
        ba.SphericalSegment(sphere_radius, 0, bottom_cup_height))
    bottom_cup = ba.Particle(
        teflon_mat,
        ba.SphericalSegment(sphere_radius, 0, sphere_radius*2 - bottom_cup_height))
    bottom_cup.rotate(ba.RotationX(180*deg))

    composition = ba.Compound()
    composition.addComponent(top_cup, R3(0, 0, bottom_cup_height))
    composition.addComponent(bottom_cup, R3(0, 0, bottom_cup_height))
    composition.translate(0, 0, -bottom_cup_height)

    vacuum_layer = ba.Layer(vacuum)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(ba.Dilute2D(0.001, composition))

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1, 0.1*nm, 0.2*deg)
    n = 100
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    return ba.ScatteringSimulation(beam, sample, detector)


if __name__ == '__main__':
    sample = get_sample()
    ba.showSample3D(sample, sample_size=120*nm, seed=0)
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_datafield(result, unit_aspect=1)
    bp.plt.show()
