#!/usr/bin/env python3
"""
GISAS of a sphere in an absorptive multilayer, with SLD materials.

Three layers: vacuum / Teflon (60 nm) / substrate.
A silver sphere (5 nm radius) sits in the vacuum layer with X, Y, Z rotations.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    middle_layer_thickness = 60 * nm

    vacuum = ba.Vacuum()
    middle_color = (0.93, 0.72, 0.25)
    middle_mat = ba.SLDMaterial("Teflon", middle_color, 4.7573e-6, 1.6724e-12)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.SLDMaterial(
        "Substrate", substrate_color, 2.0728e-6, 2.3747e-11)
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.SLDMaterial("Ag", particle_color, 3.4682e-6, 1.0309e-8)

    ff = ba.Sphere(5 * nm)
    particle = ba.Particle(particle_mat, ff)
    particle.rotate(ba.RotationZ(10 * deg))
    particle.rotate(ba.RotationY(10 * deg))
    particle.rotate(ba.RotationX(10 * deg))
    particle.translate(ba.R3(0, 0, -middle_layer_thickness / 2))

    vacuum_layer = ba.Layer(vacuum)
    middle_layer = ba.Layer(middle_mat, middle_layer_thickness)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(ba.Dilute2D(.01, particle))

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(middle_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    ba.showSample3D(sample, sample_size=120*nm, seed=0)
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_datafield(result, unit_aspect=1)
    bp.plt.show()
