//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/FormfactorPolyhedron.h
//! @brief     Defines interface FormfactorPolyhedron.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_IFORMFACTORPOLYHEDRON_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_IFORMFACTORPOLYHEDRON_H

#include "Sample/Particle/HardFormfactor.h"
#include <ff/Polyhedron.h>
#include <ff/Topology.h> // needed by all child classes
#include <memory>

namespace ff {
class IBody;
}

//! A polyhedron, for form factor computation.

class FormfactorPolyhedron : public HardFormfactor {
public:
    FormfactorPolyhedron();
    ~FormfactorPolyhedron() override;

    complex_t formfactorAtBottom(C3 q) const override;
    double volume() const override;
    std::shared_ptr<ff::Polyhedron> getPolyhedron() const;

#ifndef SWIG
    double radialExtension() const override;

    double zBottom(const Rotation* rotation) const override;
    double zTop(const Rotation* rotation) const override;

protected:
    void setPolyhedron(const ff::Topology& topology, double z_bottom,
                       const std::vector<R3>& vertices) const;

    mutable std::shared_ptr<ff::IBody> pimpl;
#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_IFORMFACTORPOLYHEDRON_H
