//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sim/Residual/ChiSquaredModule.cpp
//! @brief     Implements class ChiSquaredModule.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sim/Residual/ChiSquaredModule.h"
#include "Sim/Residual/IntensityFunction.h"
#include "Sim/Residual/VarianceFunctions.h"
#include <cmath>
#include <limits>

ChiSquaredModule::ChiSquaredModule()
    : m_variance_function(new VarianceSimFunction)
{
}

ChiSquaredModule::ChiSquaredModule(const ChiSquaredModule& other)
{
    if (other.m_variance_function)
        m_variance_function.reset(other.m_variance_function->clone());
    if (other.m_intensity_function)
        m_intensity_function.reset(other.m_intensity_function->clone());
}

ChiSquaredModule::~ChiSquaredModule() = default;

ChiSquaredModule* ChiSquaredModule::clone() const
{
    return new ChiSquaredModule(*this);
}

const VarianceFunction* ChiSquaredModule::varianceFunction() const
{
    return m_variance_function.get();
}

void ChiSquaredModule::setVarianceFunction(const VarianceFunction& variance_function)
{
    m_variance_function.reset(variance_function.clone());
}

const IntensityFunction* ChiSquaredModule::intensityFunction() const
{
    return m_intensity_function.get();
}

void ChiSquaredModule::setIntensityFunction(const IntensityFunction& intensity_function)
{
    m_intensity_function.reset(intensity_function.clone());
}

double ChiSquaredModule::residual(double a, double b)
{
    double value_simu = a;
    double value_real = b;

    if (m_intensity_function) {
        value_simu = m_intensity_function->evaluate(value_simu);
        value_real = m_intensity_function->evaluate(value_real);
    }

    double variance = m_variance_function->variance(value_real, value_simu);
    double normalize = variance <= 0 ? std::numeric_limits<double>::min() : std::sqrt(variance);

    return (value_simu - value_real) / normalize;
}
