//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Base/Type/CloneableVector.h
//! @brief     Defines and implements templated class CloneableVector.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif // SWIG
#ifndef BORNAGAIN_BASE_TYPE_CLONEABLEVECTOR_H
#define BORNAGAIN_BASE_TYPE_CLONEABLEVECTOR_H

#include <memory>
#include <vector>

//! A vector of unique pointers to objects that are cloneable.
//!
//! Equips vector<unique_ptr<T>> with copy constructor.
//! For use with polymorphic objects, or in pimpl idiom.

//! If the copy constructor or the copy assignment operator is used,
//! then there must be a function T::clone().

template <typename T> class CloneableVector : public std::vector<std::unique_ptr<T>> {
public:
    CloneableVector() = default;
    //! Constructor that takes over ownership of elements in given vector
    CloneableVector(std::vector<T*> v)
    {
        std::vector<std::unique_ptr<T>>::reserve(v.size());
        for (T* e : v)
            std::vector<std::unique_ptr<T>>::emplace_back(std::unique_ptr<T>(e));
    }
    //! Constructor that clones elements in given vector
    CloneableVector(const CloneableVector& other)
        : std::vector<std::unique_ptr<T>>()
    {
        std::vector<std::unique_ptr<T>>::reserve(other.size());
        for (const auto& e : other)
            std::vector<std::unique_ptr<T>>::emplace_back(std::unique_ptr<T>(e->clone()));
    }
    CloneableVector(CloneableVector&& other) = default;
    ~CloneableVector() = default;

    CloneableVector& operator=(const CloneableVector& other)
    {
        if (this == &other)
            return *this;
        std::vector<std::unique_ptr<T>>::clear();
        std::vector<std::unique_ptr<T>>::reserve(other.size());
        for (const auto& e : other)
            std::vector<std::unique_ptr<T>>::emplace_back(std::unique_ptr<T>(e->clone()));
        return *this;
    }
    CloneableVector& operator=(CloneableVector&& other) = default;
};

#endif // BORNAGAIN_BASE_TYPE_CLONEABLEVECTOR_H
