//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Device/Resolution/ResolutionFunction2DGaussian.h
//! @brief     Defines class ResolutionFunction2DGaussian.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_DEVICE_RESOLUTION_RESOLUTIONFUNCTION2DGAUSSIAN_H
#define BORNAGAIN_DEVICE_RESOLUTION_RESOLUTIONFUNCTION2DGAUSSIAN_H

#include "Device/Resolution/ResolutionFunction2D.h"

//! Simple gaussian two-dimensional resolution function.

class ResolutionFunction2DGaussian : public ResolutionFunction2D {
public:
    ResolutionFunction2DGaussian(double sigma_x, double sigma_y);

#ifndef SWIG
    ResolutionFunction2DGaussian* clone() const override
    {
        return new ResolutionFunction2DGaussian(m_sigma_x, m_sigma_y);
    }
    void validate() const override;

    double evaluateCDF(double x, double y) const override;

    double sigmaX() const { return m_sigma_x; }
    double sigmaY() const { return m_sigma_y; }

private:
    ResolutionFunction2DGaussian& operator=(const ResolutionFunction2DGaussian&);

    double m_sigma_x;
    double m_sigma_y;
#endif // SWIG
};

#endif // BORNAGAIN_DEVICE_RESOLUTION_RESOLUTIONFUNCTION2DGAUSSIAN_H
