//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sim/Export/ScenePositionBuilder.h
//! @brief     Declares generation of StructData::pos_* layout coordinates.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2026
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SIM_EXPORT_SCENEPOSITIONBUILDER_H
#define BORNAGAIN_SIM_EXPORT_SCENEPOSITIONBUILDER_H

class Struct;
struct StructData;

namespace SceneGeometryDetail {

//! Layer and display-patch context needed to generate one structure's layout coordinates.
struct PositionContext {
    double lateral_half_extent;
    double layer_thickness;
    double layer_z_bottom;
    double layer_z_top;
    unsigned seed;
};

//! Fills StructData::pos_* with viewer positions for one deposited, suspended, or volume structure.
void fillStructPositions(const Struct* s, const PositionContext& ctx, StructData& sd);

} // namespace SceneGeometryDetail

#endif // BORNAGAIN_SIM_EXPORT_SCENEPOSITIONBUILDER_H
