//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Multilayer/Cuvette.h
//! @brief     Defines class Cuvette.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2026
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_MULTILAYER_CUVETTE_H
#define BORNAGAIN_SAMPLE_MULTILAYER_CUVETTE_H

#include "Sample/Sample/Sample.h"

class IParticle;
class Layer;

//! A sample consisting of three vacuum layers, for SAS simulations.
//!
//! The middle layer has the specified thickness, with interfaces at ±thickness/2.
//! Particles can be added to the middle layer using Layer::fill3D() with Disordered3D.

class Cuvette : public Sample {
public:
    Cuvette(double thickness);

    //! Returns the middle layer for adding particles.
    Layer* middleLayer() { return m_middle_layer; }

#ifndef SWIG
    Cuvette* clone() const override;

private:
    Cuvette(LayerStack* outer_stack); //!< Takes ownership of given stack (used by clone)
    Layer* m_middle_layer;            //!< Non-owning pointer to the middle layer
#endif                                // SWIG
};

#endif // BORNAGAIN_SAMPLE_MULTILAYER_CUVETTE_H
