    <hr>
    <!-- Footer -->

    <script src="/js/popper.min.js"></script>
    <script src="/js/bootstrap.js"></script>

    <!-- Setting baguetteBox gallery -->
    <script src="/js/baguetteBox.js"></script>
    <script>baguetteBox.run('.galleryscg');</script>

    <script src="/js/theme.js"></script> <!-- collapse/expand tree -->
</body>
</html>
