#!/usr/bin/env python3
"""
Sawtooth (triangular) ripple grating on a 2D paracrystal lattice.

SawtoothRippleBox particles (100 x 20 x 4 nm, d=0 for symmetric
triangular profile) arranged on a BasicLattice2D (20 x 100 nm,
90 deg angle) using Paracrystal2D with Gaussian pair-distance
distributions.
"""
import math
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    vacuum = ba.Vacuum()

    # Sawtooth ripple (d=0: symmetric triangular cross-section)
    length_ripple = 100 * nm
    ff = ba.SawtoothRippleBox(length_ripple, 20 * nm, 4 * nm, 0)
    particle = ba.Particle(particle_mat, ff)

    # 2D lattice with Paracrystal2D interference
    lattice = ba.BasicLattice2D(
        20 * nm, length_ripple, math.pi / 2, 0)
    structure = ba.Paracrystal2D(
        particle, lattice, 1000 * nm, 1e4 * nm, 1e4 * nm)
    profile = ba.Profile2DGauss(150 * nm, 600 * nm, 0)
    structure.setProbabilityDistributions(profile, profile)

    # Layers
    vacuum_layer = ba.Layer(vacuum)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(structure)

    # Sample
    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    ba.showSample3D(sample, sample_size=1200*nm, seed=0)
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_datafield(result, unit_aspect=1)
    bp.plt.show()
