//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/Disordered2D.h
//! @brief     Defines class Disordered2D.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_AGGREGATE_DISORDERED2D_H
#define BORNAGAIN_SAMPLE_AGGREGATE_DISORDERED2D_H

#include "Sample/Aggregate/Struct2D.h"

//! Disordered 2D particle assembly with a given area number density.

class Disordered2D : public Struct2D {
public:
    ~Disordered2D() override;

    std::string className() const override { return "Disordered2D"; }

#ifndef SWIG
    Disordered2D* clone() const override;

    double areaNumberDensity() const override { return m_area_density; }

protected:
    Disordered2D(double area_density, const IParticle& particle);

private:
    Disordered2D(double coverage, double area_density, const IParticle& particle);

    double m_area_density;

#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_AGGREGATE_DISORDERED2D_H
