//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/Disordered3D.cpp
//! @brief     Implements class Disordered3D.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Aggregate/Disordered3D.h"
#include <stdexcept>

Disordered3D::Disordered3D(double coverage, double volume_density, const IParticle& particle)
    : Struct3D(coverage, particle)
    , m_volume_density(volume_density)
{
}

Disordered3D::Disordered3D(double volume_density, const IParticle& particle)
    : Disordered3D(1, volume_density, particle)
{
}

Disordered3D::~Disordered3D() = default;

Disordered3D* Disordered3D::clone() const
{
    throw std::runtime_error("Disordered3D::clone() should not be called directly");
}
