#!/usr/bin/env python3
"""
GISAS by cylinders with Gaussian radius distribution on a substrate.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2


def get_sample():
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-06, 2e-08)
    vacuum = ba.Vacuum()

    # Two vacuum layers, particles deposited at their interface
    layer_top = ba.Layer(vacuum)
    layer_bottom = ba.Layer(vacuum)

    distr = ba.DistributionGaussian(10*nm, 1*nm)
    for parsample in distr.distributionSamples():
        ff = ba.Cylinder(parsample.value, 5*nm)
        particle = ba.Particle(particle_mat, ff)
        density = parsample.weight * 0.001/nm2
        layer_top.deposit2D(ba.Dilute2D(density, particle))

    sample = ba.Sample()
    sample.addLayer(layer_top)
    sample.addLayer(layer_bottom)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, -1*deg, 1*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    ba.showSample3D(sample, sample_size=300*nm, seed=0)
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_datafield(result, unit_aspect=1)
    bp.plt.show()
