//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/Dilute2D.h
//! @brief     Defines class Dilute2D.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2026
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_AGGREGATE_DILUTE2D_H
#define BORNAGAIN_SAMPLE_AGGREGATE_DILUTE2D_H

#include "Sample/Aggregate/Disordered2D.h"

//! Disordered 2D particle structure with dilute approximation (no inter-particle correlations).

class Dilute2D : public Disordered2D {
public:
    Dilute2D(double area_density, const IParticle& particle);
    ~Dilute2D() override = default;

    std::string className() const final { return "Dilute2D"; }

#ifndef SWIG
    Dilute2D* clone() const override;
    double structureFactor(const R3& q) const override;
#endif
};

#endif // BORNAGAIN_SAMPLE_AGGREGATE_DILUTE2D_H
