#!/usr/bin/env python3
"""
GISAS of a stacked two-box composition embedded in a Teflon layer.

A flat box (20 × 50 × 5 nm, rotated Z) sits below a standing box
(5 × 20 × 50 nm, rotated Y). The pair is rotated 90° around Y and
placed in a 100 nm Teflon layer. Material averaging is disabled.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3

layer_thickness = 100 * nm


def get_sample():
    vacuum = ba.Vacuum()
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Ag", particle_color, 1.245e-5, 5.419e-7)
    teflon_color = (0.93, 0.72, 0.25)
    teflon_mat = ba.RefractiveMaterial("Teflon", teflon_color, 2.9e-6, 6.019e-9)
    substrate_color = (0.24, 0.50, 0.74)
    substrate_mat = ba.RefractiveMaterial(
        "Substrate2", substrate_color, 3.212e-6, 3.244e-8)

    box1_length = 20 * nm
    box1_width = 50 * nm
    box1_height = 5 * nm
    box1 = ba.Particle(particle_mat,
                       ba.Box(box1_length, box1_width, box1_height))
    box1.rotate(ba.RotationZ(90 * deg))

    box2_length = 5 * nm
    box2_width = 20 * nm
    box2_height = 50 * nm
    box2 = ba.Particle(particle_mat,
                       ba.Box(box2_length, box2_width, box2_height))
    box2.rotate(ba.RotationY(90 * deg))
    box2.translate(R3(-box2_height / 2, 0, box2_length / 2))

    composition = ba.Compound()
    composition.addComponent(box1, R3())
    composition.addComponent(box2, R3(0, 0, box1_height))
    composition.rotate(ba.RotationY(90 * deg))
    composition.translate(R3(0, 0, -layer_thickness / 2))

    vacuum_layer = ba.Layer(vacuum)
    middle_layer = ba.Layer(teflon_mat, layer_thickness)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(ba.Dilute2D(5e-4, composition))

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(middle_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1 * nm, 0.2 * deg)
    n = 200
    detector = ba.SphericalDetector(n, -2 * deg, 2 * deg, n, 0, 2 * deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    simulation.options().setUseAvgMaterials(False)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    ba.showSample3D(sample, sample_size=120*nm, seed=0)
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_datafield(result, unit_aspect=1)
    bp.plt.show()
