#!/usr/bin/env python3
"""
Mesocrystal positional disorder (Debye-Waller damping) below and above
the substrate critical angle (alpha_c ~ 0.2 deg). Below alpha_c the
scattering wavevector has a sizable imaginary qz; the DWF uses the
analytic square q.q = qx^2+qy^2+qz^2, the correct continuation of
|q|^2 to complex q.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2, R3
import matplotlib.pyplot as plt


def get_sample(sigma):
    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-05, 2e-07)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-06, 2e-07)
    vacuum = ba.Vacuum()

    # Basis particle and 3D lattice
    inner_ff = ba.Sphere(2.5*nm)
    inner_particle = ba.Particle(particle_mat, inner_ff)
    lattice = ba.Lattice3D(R3(6*nm, 0, 0), R3(0, 6*nm, 0), R3(0, 0, 6*nm))
    crystal = ba.Crystal(inner_particle, lattice, sigma)

    # Mesocrystal: shaped crystal
    outer_ff = ba.Cylinder(15*nm, 120*nm)
    outer_particle = ba.Mesocrystal(crystal, outer_ff)

    # Layers
    top_layer = ba.Layer(vacuum)
    top_layer.deposit2D(ba.Dilute2D(0.0005/nm2, outer_particle))
    top_layer.setNumberOfSlices(20)

    # Sample
    sample = ba.Sample()
    sample.addLayer(top_layer)
    sample.addLayer(ba.Layer(substrate_mat))
    return sample


def simulate(alpha_i, sigma):
    n = 200
    beam = ba.Beam(1e9, 0.1*nm, alpha_i)
    detector = ba.SphericalDetector(1, -0.01*deg, 0.01*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, get_sample(sigma), detector)
    return simulation.simulate()


if __name__ == '__main__':
    sigmas = [0*nm, 0.5*nm, 1.0*nm, 1.5*nm]
    panels = [(0.08*deg, 'evanescent'), (0.20*deg, 'propagating')]
    results = []

    _, axes = plt.subplots(1, 2, figsize=(12, 5), sharey=True)
    for ax, (alpha_i, regime) in zip(axes, panels):
        for sigma in sigmas:
            result = simulate(alpha_i, sigma)
            results.append(result)
            ax.semilogy(result.yCenters()/deg, result.flatVector(),
                        label=f'sigma = {sigma/nm:.1f} nm')
        ax.set_title(f'alpha_i = {alpha_i/deg:.2f} deg ({regime})')
        ax.set_xlabel('alpha_f (deg)')
        ax.legend()
    axes[0].set_ylabel('Intensity')
    plt.tight_layout()
    ba.showSample3D(get_sample(sigmas[0]), sample_size=220*nm, seed=0)
    plt.show()
