#!/usr/bin/env python3
"""
Specular reflectometry of a rough multilayer using the
Névot-Croce (NC) roughness model.

A 5× [PartA (2.5 nm) / PartB (5 nm)] stack on a substrate is simulated
with self-affine interface roughness and common-depth cross-correlation,
using the ErfTransient profile function (Névot-Croce model).
Compare SpecularSimulationWithRoughness.py which uses TanhTransient.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, angstrom


def get_sample():
    vacuum = ba.Vacuum()
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 15e-6, 0)
    a_color = (0.86, 0.24, 0.18)
    a_mat = ba.RefractiveMaterial("PartA", a_color, 5e-6, 0)
    b_color = (0.25, 0.65, 0.35)
    b_mat = ba.RefractiveMaterial("PartB", b_color, 10e-6, 0)

    autocorr = ba.SelfAffineFractalModel(1*nm, 0.3, 5*nm)
    crosscorr = ba.CommonDepthCrosscorrelation(10*nm)
    roughness = ba.Roughness(autocorr, ba.ErfTransient(), crosscorr)

    vacuum_layer = ba.Layer(vacuum)
    layer_a = ba.Layer(a_mat, 2.5*nm, roughness)
    layer_b = ba.Layer(b_mat, 5*nm, roughness)
    substrate_layer = ba.Layer(substrate_mat, roughness)

    n_repetitions = 5
    stack = ba.LayerStack(n_repetitions)
    stack.addLayer(layer_a)
    stack.addLayer(layer_b)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addStack(stack)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    n = <%= test_mode ? 50 : 500 %>
    scan = ba.AlphaScan(n, 2*deg/n, 2*deg)
    scan.setWavelength(1.54*angstrom)
    return ba.SpecularSimulation(scan, sample)


<%= include_partial("template/main/runsim.py",
    "ba.showSample3D(sample, sample_size=120*nm, seed=0)") -%>
