#!/usr/bin/env python3
"""
GISAS of a multilayer with linear-growth roughness autocorrelation model.

Five repetitions of PartA (5e-6, 2.5 nm) on Substrate (15e-6).
PartA roughness uses LinearGrowthModel; substrate roughness uses
SelfAffineFractalModel. Both use TanhTransient profile.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    vacuum = ba.Vacuum()
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 15e-6, 0)
    parta_color = (0.86, 0.24, 0.18)
    parta_mat = ba.RefractiveMaterial("PartA", parta_color, 5e-6, 0)

    transient = ba.TanhTransient()

    autocorr_base = ba.SelfAffineFractalModel(1*nm, 0.3, 5*nm)
    roughness_base = ba.Roughness(autocorr_base, transient)

    autocorr = ba.LinearGrowthModel(1, 1, 10, 100, 1000, 0.5)
    roughness = ba.Roughness(autocorr, transient)

    vacuum_layer = ba.Layer(vacuum)
    partA_layer = ba.Layer(parta_mat, 2.5*nm, roughness)
    substrate_layer = ba.Layer(substrate_mat, roughness_base)

    stack = ba.LayerStack(5)
    stack.addLayer(partA_layer)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addStack(stack)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1, 0.1*nm, 0.2*deg)
    n = <%= test_mode ? 25 : 100 %>
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    return ba.ScatteringSimulation(beam, sample, detector)


<%= include_partial("template/main/runsim_unitaspect.py",
    "ba.showSample3D(sample, sample_size=120*nm, seed=0)") -%>
