#!/usr/bin/env python3
"""
Polarized GISAS in transmission geometry (PM channel) of magnetic spheres.

Magnetic spheres (R = 5 nm, z-magnetisation 1e7 A/m) are embedded in a
substrate. The detector is placed below the horizon (transmission side),
and the PM spin-flip channel (+Z polariser, -Z analyser) is recorded.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    vacuum = ba.Vacuum()
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 7e-6, 1.8e-7)
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial(
        "Particle", particle_color, 2e-5, 4e-7, R3(0, 0, 1e7))

    ff = ba.Sphere(5*nm)
    particle = ba.Particle(particle_mat, ff)
    particle.translate(R3(0, 0, -10*nm))

    vacuum_layer = ba.Layer(vacuum)
    vacuum_layer.deposit2D(ba.Dilute2D(0.001, particle))
    substrate_layer = ba.Layer(substrate_mat)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 11
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, -2*deg, 0)
    beam.setPolarization(R3(0, 0, 1))
    detector.setAnalyzer(R3(0, 0, -1))
    return ba.ScatteringSimulation(beam, sample, detector)


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    from bornagain import ba_check
    ba_check.persistence_test(result)
