    <%- if test_mode -%>
    from bornagain import ba_check
    ba_check.persistence_test(results)
    <%- elsif figure_mode -%>
    plotargs = bp.parse_commandline()
    bp.plot2d_to_row(results, unit_aspect=1, **plotargs)
    bp.export(**plotargs)
    <%- else -%>
    bp.plot2d_to_row(results, unit_aspect=1)
    bp.plt.show()
    <%- end -%>
