#!/usr/bin/env python3
"""
GISAS with two particle layouts on the same layer.

Cylinders (5 × 5 nm) and triangular prisms (10 nm base, 5 nm height)
are deposited independently on the same vacuum layer at equal density.
Each layout uses Dilute2D (no interference between particles).
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    vacuum = ba.Vacuum()

    cylinder = ba.Particle(particle_mat, ba.Cylinder(5 * nm, 5 * nm))
    prism = ba.Particle(particle_mat, ba.Prism3(10 * nm, 5 * nm))

    vacuum_layer = ba.Layer(vacuum)
    vacuum_layer.deposit2D(ba.Dilute2D(0.001, cylinder))
    vacuum_layer.deposit2D(ba.Dilute2D(0.001, prism))
    substrate_layer = ba.Layer(substrate_mat)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 11
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    from bornagain import ba_check
    ba_check.persistence_test(result)
