#!/usr/bin/env python3
"""
GISAS of a core-shell box particle rotated Z then Y, embedded in a Teflon layer.

Core: Ag box (25 x 10 x 5 nm), centered vertically inside shell.
Shell: AgO2 box (50 x 20 x 10 nm).
Particle rotated 90 deg around Z then 90 deg around Y.
Embedded at midplane of a 100 nm Teflon layer on Substrate2.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    vacuum = ba.Vacuum()
    ag_color = (0.86, 0.24, 0.18)
    ag_mat = ba.RefractiveMaterial("Ag", ag_color, 1.245e-5, 5.419e-7)
    ago2_color = (0.25, 0.65, 0.35)
    ago2_mat = ba.RefractiveMaterial("AgO2", ago2_color, 8.600e-6, 3.442e-7)
    teflon_color = (0.93, 0.72, 0.25)
    teflon_mat = ba.RefractiveMaterial("Teflon", teflon_color, 2.900e-6, 6.019e-9)
    substrate_color = (0.24, 0.50, 0.74)
    substrate_mat = ba.RefractiveMaterial(
        "Substrate2", substrate_color, 3.212e-6, 3.244e-8)

    layer_thickness = 100*nm
    shell_l, shell_w, shell_h = 50*nm, 20*nm, 10*nm
    core_l, core_w, core_h = shell_l/2, shell_w/2, shell_h/2

    core = ba.Particle(ag_mat, ba.Box(core_l, core_w, core_h))
    core.translate(R3(0, 0, (shell_h - core_h)/2))
    shell = ba.Particle(
        ago2_mat, ba.Box(shell_l, shell_w, shell_h))

    coreshell = ba.CoreAndShell(core, shell)
    coreshell.rotate(ba.RotationZ(90*deg))
    coreshell.rotate(ba.RotationY(90*deg))
    coreshell.translate(R3(0, 0, -layer_thickness/2))

    vacuum_layer = ba.Layer(vacuum)
    middle_layer = ba.Layer(teflon_mat, layer_thickness)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(ba.Dilute2D(0.001, coreshell))

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(middle_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1, 0.1*nm, 0.2*deg)
    n = 100
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    return ba.ScatteringSimulation(beam, sample, detector)


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    plotargs = bp.parse_commandline()
    bp.plot_datafield(result, unit_aspect=1, **plotargs)
    bp.export(**plotargs)
