//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/Dilute2D.cpp
//! @brief     Implements class Dilute2D.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2026
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Aggregate/Dilute2D.h"

Dilute2D::Dilute2D(double area_density, const IParticle& particle)
    : Disordered2D(area_density, particle)
{
}

Dilute2D* Dilute2D::clone() const
{
    auto* result = new Dilute2D(areaNumberDensity(), *particle());
    Struct::instructClone(result);
    return result;
}

double Dilute2D::structureFactor(const R3&) const
{
    return 1.0; // Dilute approximation: no inter-particle correlations
}
