#!/usr/bin/env python3
"""
GISAS by a a distribution of dlayouterently oriented
square lattices of cylinders on a substrate.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    vacuum = ba.Vacuum()
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)

    toplayer = ba.Layer(vacuum)
    substrate = ba.Layer(substrate_mat)

    ff = ba.Cylinder(3*nm, 4*nm)
    particle = ba.Particle(particle_mat, ff)

    distr = ba.DistributionGate(0*deg, 90*deg)
    distr.setNSamples(21)
    for parsample in distr.distributionSamples():
        layout = ba.Crystal2D(particle, ba.SquareLattice2D(25*nm, parsample.value))
        layout.setDecayFunction(ba.Profile2DCauchy(100*nm, 100*nm, 0))
        toplayer.addDeposit2D(parsample.weight, layout)

    sample = ba.Sample()
    sample.addLayer(toplayer)
    sample.addLayer(substrate)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = <%= test_mode ? 11 : 100 %>
    detector = ba.SphericalDetector(n, -1*deg, 1*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


<%= include_partial("template/main/runsim_unitaspect.py",
    "ba.showSample3D(sample, sample_size=300*nm, seed=0)") -%>
