//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sim/Export/SceneParticleCollector.h
//! @brief     Declares flattening of IParticle trees into StructData particle templates.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2026
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SIM_EXPORT_SCENEPARTICLECOLLECTOR_H
#define BORNAGAIN_SIM_EXPORT_SCENEPARTICLECOLLECTOR_H

#include "Base/Vector/RotMatrix.h"
#include <heinz/Vectors3D.h>

class IParticle;
struct StructData;

namespace SceneGeometryDetail {

//! Appends ParticleData templates and Mixture selection metadata for one structure's particle tree.
void collectParticleTemplates(const IParticle* ipart, const R3& base_pos, const RotMatrix& base_rot,
                              double opacity, StructData& sd);

} // namespace SceneGeometryDetail

#endif // BORNAGAIN_SIM_EXPORT_SCENEPARTICLECOLLECTOR_H
