//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Slice/ProfileUtil.h
//! @brief     Defines class ProfileHelper.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif // SWIG
#ifndef BORNAGAIN_RESAMPLE_SLICE_PROFILEUTIL_H
#define BORNAGAIN_RESAMPLE_SLICE_PROFILEUTIL_H

#include <heinz/Complex.h>
#include <memory>
#include <utility>
#include <vector>

class Slice;

//! Functions that generate the material profile of a sample as a function of depth.
//!
//! The generated profile contains the complex SLD for SLD materials and the parameters
//! delta and beta for refractive index materials

namespace ProfileUtil {

std::vector<complex_t> sldProfile(const std::vector<std::unique_ptr<const Slice>>& stack,
                                  const std::vector<double>& z_values);

std::vector<double> magnetizationProfile(const std::vector<std::unique_ptr<const Slice>>& stack,
                                         const std::vector<double>& z_values, std::string xyz);
std::pair<double, double> defaultLimits(const std::vector<std::unique_ptr<const Slice>>& stack);

} // namespace ProfileUtil

#endif // BORNAGAIN_RESAMPLE_SLICE_PROFILEUTIL_H
