//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Device/Resolution/DetectorResolution.h
//! @brief     Defines interface DetectorResolution.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_DEVICE_RESOLUTION_IDETECTORRESOLUTION_H
#define BORNAGAIN_DEVICE_RESOLUTION_IDETECTORRESOLUTION_H

#include "Base/Type/Cloneable.h"

class Datafield;

//! Interface for detector resolution algorithms

class DetectorResolution : public Cloneable {
public:
    virtual ~DetectorResolution() = default;

#ifndef SWIG
    DetectorResolution* clone() const override = 0;
#endif // SWIG

    //! Apply the resolution function to the intensity data
    virtual void execDetectorResolution(Datafield* p_intensity_map) const = 0;
};

#endif // BORNAGAIN_DEVICE_RESOLUTION_IDETECTORRESOLUTION_H
