In {{ range $index, $c :=  .Params.categories }}
  {{- if eq $index 0 -}}
    <a href="/categories/{{ urlize . }}">{{ title . }}</a>
  {{- else -}}
    , <a href="/categories/{{ urlize . }}">{{ title . }}</a>
  {{- end -}}
{{ end }}
