#!/usr/bin/env python3
"""
Unpolarized GISAS of a rotated magnetic sphere.

The same sample as MagneticRotationZPM (sphere with x-oriented
magnetisation rotated 20° around Z, substrate with y-oriented
magnetisation) simulated without polariser or analyser. Tests the scalar
computation path with a magnetic sample.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    vacuum = ba.Vacuum()
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 7e-6, 2e-8,
                                                R3(0, 1e6, 0))
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("MagParticle", particle_color, 6e-4, 2e-8,
                                               R3(1e6, 0, 0))

    ff = ba.Sphere(5*nm)
    particle = ba.Particle(particle_mat, ff)
    particle.rotate(ba.RotationZ(20*deg))

    vacuum_layer = ba.Layer(vacuum)
    substrate_layer = ba.Layer(substrate_mat)
    substrate_layer.suspend2D(ba.Dilute2D(0.001, particle))

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    ba.showSample3D(sample, sample_size=120*nm, seed=0)
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_datafield(result, unit_aspect=1)
    bp.plt.show()
