#!/usr/bin/env python3
"""
GISAS of cylinders on a square lattice rotated by 30 degrees.

The 10 nm square lattice and its Cauchy decay function are both
rotated by 30 degrees, breaking the x/y symmetry of the diffraction
pattern.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm
import math


def get_sample():
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    vacuum = ba.Vacuum()

    ff = ba.Cylinder(5 * nm, 5 * nm)
    particle = ba.Particle(particle_mat, ff)

    lattice = ba.SquareLattice2D(10 * nm, 30 * deg)
    structure = ba.Crystal2D(particle, lattice)
    profile = ba.Profile2DCauchy(300 / (2 * math.pi) * nm,
                                 100 / (2 * math.pi) * nm, 30 * deg)
    structure.setDecayFunction(profile)

    vacuum_layer = ba.Layer(vacuum)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(structure)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    plotargs = bp.parse_commandline()
    bp.plot_datafield(result, unit_aspect=1, **plotargs)
    bp.export(**plotargs)
