//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/Disordered2D.cpp
//! @brief     Implements class Disordered2D.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Aggregate/Disordered2D.h"
#include <stdexcept>

Disordered2D::Disordered2D(double coverage, double area_density, const IParticle& particle)
    : Struct2D(coverage, particle)
    , m_area_density(area_density)
{
}

Disordered2D::Disordered2D(double area_density, const IParticle& particle)
    : Disordered2D(1, area_density, particle)
{
}

Disordered2D::~Disordered2D() = default;

Disordered2D* Disordered2D::clone() const
{
    throw std::runtime_error("Disordered2D::clone() should not be called directly");
}
