//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sim/Background/ConstantBackground.h
//! @brief     Defines class ConstantBackground.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SIM_BACKGROUND_CONSTANTBACKGROUND_H
#define BORNAGAIN_SIM_BACKGROUND_CONSTANTBACKGROUND_H

#include "Sim/Background/Background.h"

//! Constant background, to be added to simulated scattering.

class ConstantBackground : public Background {
public:
    ConstantBackground(double background_value);

    double perturbed(double intensity) const override;

#ifndef SWIG
    ConstantBackground* clone() const override;
    void validate() const override;

    double backgroundValue() const { return m_background_value; }

private:
    double m_background_value;
#endif // SWIG
};

#endif // BORNAGAIN_SIM_BACKGROUND_CONSTANTBACKGROUND_H
