//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/CosineRipple.h
//! @brief     Defines classes CosineRipple*.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_COSINERIPPLE_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_COSINERIPPLE_H

#include "Sample/HardParticle/ProfileRipple.h"

//! The form factor for a cosine ripple, with box profile in elongation direction.
class CosineRippleBox : public CosineRipple {
public:
    CosineRippleBox(double length, double width, double height);

#ifndef SWIG
    CosineRippleBox* clone() const override;
#endif // SWIG

    std::string className() const final { return "CosineRippleBox"; }

    bool contains(const R3& position) const override;

private:
    complex_t factor_x(complex_t qx) const override;
};

//! The form factor for a cosine ripple, with Gaussian profile in elongation direction.
class CosineRippleGauss : public CosineRipple {
public:
    CosineRippleGauss(double length, double width, double height);

#ifndef SWIG
    CosineRippleGauss* clone() const override;
#endif // SWIG

    std::string className() const final { return "CosineRippleGauss"; }

    bool contains(const R3& position) const override;

private:
    complex_t factor_x(complex_t qx) const override;
};

//! The form factor for a cosine ripple, with Lorentz form factor in elongation direction.
class CosineRippleLorentz : public CosineRipple {
public:
    CosineRippleLorentz(double length, double width, double height);

#ifndef SWIG
    CosineRippleLorentz* clone() const override;
#endif // SWIG

    std::string className() const final { return "CosineRippleLorentz"; }

    bool contains(const R3& position) const override;

private:
    complex_t factor_x(complex_t qx) const override;
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_COSINERIPPLE_H
