//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/LongBoxLorentz.cpp
//! @brief     Implements class LongBoxLorentz.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/HardParticle/LongBoxLorentz.h"
#include "Base/Math/Functions.h"
#include "Sample/Shape/BoxNet.h"

LongBoxLorentz::LongBoxLorentz(double length, double width, double height)
    : HardFormfactor()
    , m_length(length)
    , m_width(width)
    , m_height(height)
{
    LongBoxLorentz::validate();
}

complex_t LongBoxLorentz::formfactorAtBottom(C3 q) const
{
    complex_t qxL2 = 2.5 * std::pow(m_length * q.x(), 2);
    complex_t qyWdiv2 = m_width * q.y() / 2.0;
    complex_t qzHdiv2 = m_height * q.z() / 2.0;

    return m_height * m_length * m_width * std::exp(complex_t(0., 1.) * qzHdiv2) / (1.0 + qxL2)
           * Math::sinc(qyWdiv2) * Math::sinc(qzHdiv2);
}

void LongBoxLorentz::validate() const
{
    if (m_length <= 0)
        throw std::runtime_error("length <= 0");
    if (m_width <= 0)
        throw std::runtime_error("width <= 0");
    if (m_height <= 0)
        throw std::runtime_error("height <= 0");

    m_shape3D = std::make_unique<BoxNet>(m_length, m_width, m_height);

    m_validated = true;
}

bool LongBoxLorentz::contains(const R3&) const
{
    throw std::runtime_error("LongBoxLorentz cannot be used as mesocrystal outer shape");
}
