#!/usr/bin/env python3
"""
Core shell nanoparticles.
Alternative implementation using particle composition
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    """
    A sample with box-shaped core-shell particles on top of air.
    """

    # Materials
    diff_color = (0.86, 0.24, 0.18)
    diff_mat = ba.RefractiveMaterial("Core", diff_color, -4e-6, 0)
    shell_color = (0.25, 0.65, 0.35)
    shell_mat = ba.RefractiveMaterial("Shell", shell_color, 1e-5, 2e-8)

    # Form factors
    ff_1 = ba.Box(12*nm, 12*nm, 7*nm)
    ff_2 = ba.Box(16*nm, 16*nm, 8*nm)

    # Particles
    core = ba.Particle(diff_mat, ff_1)
    shell = ba.Particle(shell_mat, ff_2)
    particle = ba.Compound()
    particle.addComponent(shell)
    particle.addComponent(core, R3(0, 0, 0.5*nm))

    # Sample with particles in vacuum over vacuum substrate
    layer_1 = ba.Layer(ba.Vacuum())
    layer_1.deposit2D(ba.Dilute2D(0.001, particle))
    layer_2 = ba.Layer(ba.Vacuum())

    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, 0., 2*deg, n, 0., 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    plotargs = bp.parse_commandline()
    bp.plot_datafield(result, unit_aspect=1, **plotargs)
    bp.export(**plotargs)
