//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/LongBoxGauss.h
//! @brief     Defines class LongBoxGauss.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_LONGBOXGAUSS_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_LONGBOXGAUSS_H

#include "Sample/Particle/HardFormfactor.h"

//! The form factor for a long rectangular box.

class LongBoxGauss : public HardFormfactor {
public:
    LongBoxGauss(double length, double width, double height);

    std::string className() const final { return "LongBoxGauss"; }

#ifndef SWIG
    LongBoxGauss* clone() const override { return new LongBoxGauss(m_length, m_width, m_height); }
#endif // SWIG

    void validate() const override;

    double length() const { return m_length; }
    double height() const { return m_height; }
    double width() const { return m_width; }
    double radialExtension() const override { return m_length / 2.0; }

    complex_t formfactorAtBottom(C3 q) const override;


    bool contains(const R3& position) const override;

private:
    double m_length;
    double m_width;
    double m_height;
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_LONGBOXGAUSS_H
