//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sim/Background/Background.h
//! @brief     Defines interface Background.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SIM_BACKGROUND_IBACKGROUND_H
#define BORNAGAIN_SIM_BACKGROUND_IBACKGROUND_H

#include "Base/Type/Cloneable.h"

//! Abstract base class for background noise, to be added to simulated scattering.
//!
//! Base class of ConstantBackground, PoissonBackground.

class Background : public Cloneable {
public:
    Background() = default;
    virtual ~Background() = default;

    virtual double perturbed(double element) const = 0;

#ifndef SWIG
    Background* clone() const override = 0;

    virtual void validate() const {}

protected:
    mutable bool m_validated{false};
#endif // SWIG
};

#endif // BORNAGAIN_SIM_BACKGROUND_IBACKGROUND_H
