//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/HorizontalCylinder.h
//! @brief     Defines class Cylinder.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_HORIZONTALCYLINDER_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_HORIZONTALCYLINDER_H

#include "Sample/Particle/HardFormfactor.h"

//! A circular cylinder.

class HorizontalCylinder : public HardFormfactor {
public:
    HorizontalCylinder(double radius, double length, double rm_top, double rm_bottom);
    HorizontalCylinder(double radius, double length);

    complex_t formfactorAtBottom(C3 q) const override;

    std::string className() const final { return "HorizontalCylinder"; }

#ifndef SWIG
    HorizontalCylinder* clone() const override
    {
        return new HorizontalCylinder(m_radius, m_length, m_rm_top, m_rm_bottom);
    }
    void validate() const override;

    double radius() const { return m_radius; }
    double length() const { return m_length; }
    double cutFromTop() const { return m_rm_top; }
    double cutFromBottom() const { return m_rm_bottom; }

    double radialExtension() const override { return m_radius; }
    bool contains(const R3& position) const override;

private:
    double m_radius;
    double m_length;
    double m_rm_top;    //!< height of removed top cap
    double m_rm_bottom; //!< height of removed bottom cap
#endif                  // SWIG
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_HORIZONTALCYLINDER_H
