#!/usr/bin/env python3
"""
GISAS simulation including the specular peak, for cylinders
on a square 2D lattice.

Same sample as SquareLattice2D.py (Crystal2D, 10 nm spacing,
5 nm cylinders, Cauchy decay). Specular peak is enabled via
simulation.options().setIncludeSpecular(True).
"""
from math import pi
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    vacuum = ba.Vacuum()

    # Particles
    ff = ba.Cylinder(5 * nm, 5 * nm)
    particle = ba.Particle(particle_mat, ff)

    # Square lattice with Crystal2D and Cauchy decay
    lattice = ba.SquareLattice2D(10 * nm, 0)
    layout = ba.Crystal2D(particle, lattice)
    profile = ba.Profile2DCauchy(
        300 / (2 * pi) * nm, 100 / (2 * pi) * nm, 0)
    layout.setDecayFunction(profile)

    # Layers
    layer_1 = ba.Layer(vacuum)
    layer_1.deposit2D(layout)
    layer_2 = ba.Layer(substrate_mat)

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1 * nm, 0.2 * deg)
    n = 11
    detector = ba.SphericalDetector(n, -2 * deg, 2 * deg, n, 0, 2 * deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    simulation.options().setIncludeSpecular(True)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    from bornagain import ba_check
    ba_check.persistence_test(result)
