<div class="container next-prev">
<nav>
  <ul class="pagination pagination-sm justify-content-end">
{{- $currentNode := . -}}
{{- template "menu-nextprev" dict "menu" .Site.Home "currentnode" $currentNode -}}

{{- define "menu-nextprev" -}}
    {{- $currentNode := .currentnode -}}
    {{- if ne .menu.Params.hidden true -}}
        {{- if hasPrefix $currentNode.Permalink .menu.Permalink -}}
            {{- $currentNode.Scratch.Set "NextPageOK" "OK" -}}
            {{- $currentNode.Scratch.Set "prevPage" ($currentNode.Scratch.Get "prevPageTmp") -}}
        {{- else -}}
            {{- if eq ($currentNode.Scratch.Get "NextPageOK") "OK" -}}
                {{- $currentNode.Scratch.Set "NextPageOK" nil -}}
                {{- $currentNode.Scratch.Set "nextPage" .menu -}}
            {{- end -}}
        {{- end -}}
        {{- $currentNode.Scratch.Set "prevPageTmp" .menu -}}

            {{- $currentNode.Scratch.Set "pages" .menu.Pages -}}
            {{- if .menu.IsHome -}}
                {{- $currentNode.Scratch.Set "pages" .menu.Sections -}}
            {{- else if .menu.Sections -}}
                {{- $currentNode.Scratch.Set "pages" (.menu.Pages | union .menu.Sections) -}}
            {{- end -}}
            {{- $pages := ($currentNode.Scratch.Get "pages") -}}

            {{- range $pages.ByWeight -}}
                {{- template "menu-nextprev" dict "menu" . "currentnode" $currentNode -}}
            {{- end -}}
    {{- end -}}
{{- end -}}


{{- if not $.Site.Params.disableNavChevron -}}
    {{- with ($.Scratch.Get "prevPage") -}}
    <li class="page-item ">
      <a href="{{.Permalink}}" class="page-link"><i class="fa fa-chevron-left fa-1g "></i>
      </a>
    </li>

    {{ end -}}
    {{- with ($.Scratch.Get "nextPage") -}}
    <li class="page-item ">
      <a href="{{.Permalink}}" class="page-link"><i class="fa fa-chevron-right fa-1g "></i>
      </a>
    </li>
    {{- end }}
{{- end -}}
</ul>
</nav>
</div>
