//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Particle/PolyhedralUtil.h
//! @brief     Defines interface IDecoratableBorn.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif // SWIG
#ifndef BORNAGAIN_SAMPLE_PARTICLE_POLYHEDRALUTIL_H
#define BORNAGAIN_SAMPLE_PARTICLE_POLYHEDRALUTIL_H

#include <heinz/Vectors3D.h>
#include <vector>

class Rotation;

namespace PolyhedralUtil {

//! Returns minimum z-coordinate of the vertices after rotation
double zMin(const std::vector<R3>& vertices, const Rotation* rotation);

//! Returns maximum z-coordinate of the vertices after rotation
double zMax(const std::vector<R3>& vertices, const Rotation* rotation);

} // namespace PolyhedralUtil

#endif // BORNAGAIN_SAMPLE_PARTICLE_POLYHEDRALUTIL_H
