/* Styles for the gallery */

.tz-gallery {
  padding-top: 1rem;
  padding-bottom: 1rem;  
}

.tz-gallery figure {
overflow: visible!important;
margin-bottom: 1rem;
}

.tz-gallery .thumbnail {
    padding: 0;
    margin-bottom: 30px;
    border: none;
}

.tz-gallery img {
    border-radius: 3px;
}

.tz-gallery .caption{
    padding: 26px 30px;
    text-align: center;
}

.tz-gallery .caption p {
    font-size: 12px;
    color: #7b7d7d;
    margin: 0;
}

.tz-gallery .lightbox img {
    width: 100%;
    margin-bottom: 5px;
    transition: 0.2s ease-in-out;
    box-shadow: 0 2px 3px rgba(0,0,0,0.2);
}

.tz-gallery .lightbox img:hover {
    transform: scale(1.01);
    box-shadow: 0 8px 15px rgba(0,0,0,0.3);
}


.baguetteBox-button {
    background-color: transparent !important;
}


@media(max-width: 768px) {
    body {
        padding: 0;
    }

    .container.gallery-container {
        border-radius: 0;
    }
}