//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sim/Background/ConstantBackground.cpp
//! @brief     Implements class ConstantBackground.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sim/Background/ConstantBackground.h"
#include "Base/Util/Assert.h"

ConstantBackground::ConstantBackground(double background_value)
    : Background()
    , m_background_value(background_value)
{
    ConstantBackground::validate();
}

double ConstantBackground::perturbed(double intensity) const
{
    ASSERT(m_validated);
    return intensity + m_background_value;
}

ConstantBackground* ConstantBackground::clone() const
{
    return new ConstantBackground(m_background_value);
}

void ConstantBackground::validate() const
{
    if (m_background_value < 0)
        throw std::runtime_error("background_value");

    m_validated = true;
}
