#!/usr/bin/env python3
"""
GISAS of a larger cylindrical mesocrystal on a substrate.

Spheres (4 nm radius) on a 10 nm cubic lattice fill a cylinder of
diameter and height 28 nm (3×3×3 unit cells). Compare Mesocrystal.py
which uses smaller parameters.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    vacuum = ba.Vacuum()

    n = 3
    L = 10 * nm
    R = 4 * nm

    inner_ff = ba.Sphere(R)
    inner_particle = ba.Particle(particle_mat, inner_ff)

    lattice = ba.Lattice3D(R3(L, 0, 0), R3(0, L, 0), R3(0, 0, L))
    crystal = ba.Crystal(inner_particle, lattice)

    size = L * (n - 1) + 2 * R
    outer_ff = ba.Cylinder(size / 2, size)
    meso = ba.Mesocrystal(crystal, outer_ff)

    vacuum_layer = ba.Layer(vacuum)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(ba.Dilute2D(0.001, meso))

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    ba.showSample3D(sample, sample_size=250*nm, seed=0)
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_datafield(result, unit_aspect=1)
    bp.plt.show()
