#!/usr/bin/env python3
"""
Cylindrical mesocrystal on a substrate.
Calculation using exact computation and 
an approximate algorithm of the Fourier sum with a cutoff.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2, R3


def get_sample():
    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-05, 2e-08)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-06, 2e-08)
    vacuum = ba.Vacuum()

    # Basis particle
    inner_ff = ba.Sphere(3*nm)
    inner_particle = ba.Particle(particle_mat, inner_ff)

    # 3D lattice
    lattice = ba.Lattice3D(R3(8*nm, 0, 0), R3(0, 8*nm, 0),
                           R3(0, 0, 8*nm))

    # Crystal
    crystal = ba.Crystal(inner_particle, lattice)

    # Mesocrystal: shaped crystal
    outer_ff = ba.Cylinder(20*nm, 50*nm)
    outer_particle = ba.Mesocrystal(crystal, outer_ff)

    # Layers
    layer_1 = ba.Layer(vacuum)
    layer_1.deposit2D(ba.Dilute2D(0.0001/nm2, outer_particle))
    layer_2 = ba.Layer(substrate_mat)

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)

    return sample


<%= include_partial("template/sim/gisas_standard_wide.py") -%>


def simulate(title, use_fourier):
    sample = get_sample()
    simulation = get_simulation(sample)
    if use_fourier:
        simulation.options().setMesoReciprocalSum(True, 2.5)
    result = simulation.simulate()
    result.setTitle(title)
    return result


if __name__ == '__main__':
    <%- unless test_mode || figure_mode -%>
    ba.showSample3D(get_sample(), sample_size=250*nm, seed=0)
    <%- end -%>
    results = [
        simulate('exact sum', False),
        simulate('Fourier sum', True),
    ]
<%= include_partial("template/main/plot_row_unitaspect.py") -%>
