#!/usr/bin/env python3
"""
Cylinders forming an infinite 2D square lattice
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    """
    A sample with cylinders on a substrate, forming an infinite 2D square lattice
    """

    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 0.0006, 2e-08)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-06, 2e-08)
    vacuum = ba.Vacuum()

    # Particles
    ff = ba.Cylinder(3*nm, 3*nm)
    particle = ba.Particle(particle_mat, ff)

    # 2D lattice
    lattice = ba.BasicLattice2D(25*nm, 25*nm, 90*deg, 30*deg)
    layout = ba.Crystal2D(particle, lattice)
    profile = ba.Profile2DCauchy(1000*nm, 1000*nm, 0*deg)
    layout.setDecayFunction(profile)

    # Layers
    layer_1 = ba.Layer(vacuum)
    layer_1.deposit2D(layout)
    layer_2 = ba.Layer(substrate_mat)

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)

    return sample


<%= include_partial("template/sim/gisas_standard_wide.py") -%>


<%= include_partial("template/main/runsim_unitaspect.py",
    "ba.showSample3D(sample, sample_size=300*nm, seed=0)") -%>
