//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/SphericalSegment.cpp
//! @brief     Implements class SphericalSegment.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/HardParticle/SphericalSegment.h"
#include "Base/Math/Bessel.h"
#include "Base/Math/Integrator.h"
#include "Base/Util/Assert.h"
#include "Sample/Shape/TruncatedEllipsoidNet.h"
#include <limits>
#include <numbers>

using std::numbers::pi;

SphericalSegment::SphericalSegment(double radius, double top_cut, double bottom_cut)
    : HardFormfactor()
    , m_radius(radius)
    , m_rm_top(top_cut)
    , m_rm_bottom(bottom_cut)
{
    SphericalSegment::validate();
}

//! Complex form factor.
complex_t SphericalSegment::formfactorAtBottom(C3 q) const
{
    ASSERT(m_validated);

    double untruncated_height = 2 * m_radius - m_rm_bottom;

    if (std::abs(q.mag()) < std::numeric_limits<double>::epsilon())
        return pi / 3.
               * (untruncated_height * untruncated_height * (3. * m_radius - untruncated_height)
                  - m_rm_top * m_rm_top * (3. * m_radius - m_rm_top));

    const complex_t Q2 = std::sqrt(q.x() * q.x() + q.y() * q.y()); // NOT the modulus!
    const double R2 = m_radius * m_radius;

    complex_t integral = ComplexIntegrator().integrate(
        [=](double Z) {
            const double R2Z2 = R2 - Z * Z;
            return R2Z2 * Math::Bessel::J1c(Q2 * sqrt(R2Z2)) * exp_I(q.z() * Z);
        },
        -m_radius + m_rm_bottom, m_radius - m_rm_top);
    return (2 * pi) * integral * exp_I(q.z() * (m_radius - m_rm_bottom));
}

void SphericalSegment::validate() const
{
    if (m_radius <= 0)
        throw std::runtime_error("radius <= 0");
    if (m_rm_top < 0)
        throw std::runtime_error("rm_top < 0");
    if (m_rm_bottom < 0)
        throw std::runtime_error("rm_bottom < 0");
    if (m_rm_top > (2 * m_radius - m_rm_bottom))
        throw std::runtime_error("parameters violate condition rm_top > H");
    if (m_rm_bottom > (2 * m_radius - m_rm_top))
        throw std::runtime_error("parameters violate condition rm_bottom > H");

    double untruncated_height = 2 * m_radius - m_rm_bottom;
    m_shape3D = std::make_unique<TruncatedEllipsoidNet>(m_radius, m_radius, m_radius,
                                                        untruncated_height, m_rm_top);

    m_validated = true;
}

bool SphericalSegment::contains(const R3& position) const
{
    double R = radius();
    double H = 2 * R - cutFromBottom();
    double deltaH = cutFromTop();

    if (std::abs(position.x()) > R || std::abs(position.y()) > R || position.z() < 0
        || position.z() > (H - deltaH))
        return false;

    if (std::pow(position.x() / R, 2) + std::pow(position.y() / R, 2)
            + std::pow((position.z() - (H - R)) / R, 2)
        <= 1)
        return true;

    return false;
}
