//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/Struct2D.h
//! @brief     Defines class Struct2D.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_AGGREGATE_STRUCT2D_H
#define BORNAGAIN_SAMPLE_AGGREGATE_STRUCT2D_H

#include "Sample/Aggregate/Struct.h"
#include <heinz/Vectors3D.h>

class Struct2D : public Struct {
#ifndef SWIG
public:
    Struct2D* clone() const override = 0;

    //! Returns the number of particles per unit area (1/nm²)
    virtual double areaNumberDensity() const = 0;

    //! Structure factor for a given wavevector transfer.
    virtual double structureFactor(const R3& q) const = 0;

    double scalarScattering(const DiffuseElement& ele) const override;
    double polarizedScattering(const DiffuseElement& ele) const override;

protected:
    Struct2D(double coverage, const IParticle& particle)
        : Struct(coverage, particle)
    {
    }

#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_AGGREGATE_STRUCT2D_H
