#!/usr/bin/env python3
"""
Spheres in a finite layer. For testing the slicing machinery.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2


def get_sample():
    # defining materials
    vacuum = ba.Vacuum()
    layer1_color = (0.93, 0.72, 0.25)
    layer1_mat = ba.RefractiveMaterial("Layer1", layer1_color, 4e-6, 2e-8)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 3e-5, 2e-8)

    # particle
    ff = ba.SphericalSegment(5*nm, 0, 5*nm)
    particle = ba.Particle(particle_mat, ff)

    # layers
    layer1 = ba.Layer(layer1_mat, 6*nm)
    layer1.setNumberOfSlices(10)
    layer1.deposit2D(ba.Dense2D(.002/nm2, particle))
    substrate = ba.Layer(substrate_mat)

    sample = ba.Sample()
    sample.addLayer(ba.Layer(vacuum))
    sample.addLayer(layer1)
    sample.addLayer(substrate)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 11
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    from bornagain import ba_check
    ba_check.persistence_test(result)
