#!/usr/bin/env python3
"""
Polarized GISAS (PP and MP channels) of magnetic spheres in a magnetic
layer.

Spheres with x-oriented magnetisation (1e6 A/m) are deposited on a
magnetic substrate. Two polarisation channels are shown: PP (+Z/+Z, no
spin flip) and MP (-Z/+Z, spin flip from below).
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    vacuum = ba.Vacuum()
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 7e-6, 2e-8)
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("MagParticle", particle_color, 6e-4, 2e-8,
                                               R3(1e6, 0, 0))

    ff = ba.Sphere(5*nm)
    particle = ba.Particle(particle_mat, ff)

    vacuum_layer = ba.Layer(vacuum)
    vacuum_layer.deposit2D(ba.Dilute2D(0.001, particle))
    substrate_layer = ba.Layer(substrate_mat)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample, pol_dir, an_dir):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    beam.setPolarization(pol_dir)
    detector.setAnalyzer(an_dir)
    return ba.ScatteringSimulation(beam, sample, detector)


def simulate(pol_dir, an_dir):
    sample = get_sample()
    simulation = get_simulation(sample, pol_dir, an_dir)
    return simulation.simulate()


if __name__ == '__main__':
    z_up = R3(0, 0, 1)
    z_dn = R3(0, 0, -1)
    results = [
        simulate(z_up, z_up),   # PP
        simulate(z_dn, z_up),   # MP
    ]
    ba.showSample3D(get_sample(), sample_size=120*nm, seed=0)
    bp.plot2d_to_row(results, unit_aspect=1)
    bp.plt.show()
