#!/usr/bin/env python3
"""
Cylinder form factor in DWBA with background/noise
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    from bornagain import std_samples
    return std_samples.cylinders()


def get_simulation(sample, background):
    beam = ba.Beam(1e8, 0.1*nm, 0.2*deg)
    n = 100
    detector = ba.SphericalDetector(n, 0., 2*deg, n, 0., 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    simulation.setBackground(background)
    return simulation


def simulate(background, title):
    sample = get_sample()
    simulation = get_simulation(sample, background)
    result = simulation.simulate()
    result.setTitle(title)
    return result


if __name__ == '__main__':    
    results = [
        simulate(ba.ConstantBackground(1e0), "Constant background"),
        simulate(ba.PoissonBackground(), "Poisson noise"),
    ]    
    plotargs = bp.parse_commandline()
    plotargs.update({'log_range': 5, 'unit_aspect': 1})
    bp.plot2d_to_row(results, **plotargs)
    bp.export(**plotargs)
