//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/Sphere.h
//! @brief     Defines class Sphere.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_SPHERE_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_SPHERE_H

#include "Sample/Particle/HardFormfactor.h"

//! A full sphere.

class Sphere : public HardFormfactor {
public:
    Sphere(double radius);

    complex_t formfactorAtBottom(C3 q) const override;

    std::string className() const final { return "Sphere"; }

#ifndef SWIG
    Sphere* clone() const override { return new Sphere(m_radius); }
    void validate() const override;

    double radius() const { return m_radius; }
    double height() const { return 2 * m_radius; }

    double radialExtension() const override { return m_radius; }

    double zBottom(const Rotation* rotation) const override;
    double zTop(const Rotation* rotation) const override;

    bool contains(const R3& position) const override;

protected:
    bool canSliceAnalytically(const Rotation*) const override { return true; }

private:
    double m_radius;
#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_SPHERE_H
