//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/SoftParticle/GaussSphere.h
//! @brief     Defines class GaussSphere.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_SOFTPARTICLE_GAUSSSPHERE_H
#define BORNAGAIN_SAMPLE_SOFTPARTICLE_GAUSSSPHERE_H

#include "Sample/Particle/SoftFormfactor.h"

//! The form factor of a Gaussian sphere.

class GaussSphere : public SoftFormfactor {
public:
    GaussSphere(double mean_radius);

    std::string className() const final { return "GaussSphere"; }

#ifndef SWIG
    GaussSphere* clone() const override { return new GaussSphere(m_mean_radius); }
    void validate() const override;

    double meanRadius() const { return m_mean_radius; }

    double radialExtension() const override { return m_mean_radius; }

    complex_t formfactorAtBottom(C3 q) const override;

private:
    double m_mean_radius;
    void initialize();
#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_SOFTPARTICLE_GAUSSSPHERE_H
