#!/usr/bin/env python3
"""
Dilute soft spheres (FuzzySphere) in a thick film, 3D distribution.

Soft particles have diffuse boundaries modelled by a Debye-Waller factor.
Slicing is not supported for soft particles; material averaging is disabled.

The 3D distribution uses fill3D(Dilute3D).
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm3


def get_sample():
    R = 5*nm
    thickness = 12*R  # 60 nm

    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-04, 2e-08)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-06, 2e-08)
    vacuum = ba.Vacuum()

    # Soft particle (FuzzySphere with Gaussian fuzziness), centered alignment
    ff = ba.FuzzySphere(R, 0.5*nm)
    particle = ba.Particle(particle_mat, ff)

    # Layers: vacuum / thick film / substrate
    layer_vacuum = ba.Layer(vacuum)
    layer_film = ba.Layer(vacuum, thickness)
    layer_film.fill3D(ba.Dilute3D(5e-4/nm3, particle))
    layer_substrate = ba.Layer(substrate_mat)

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_vacuum)
    sample.addLayer(layer_film)
    sample.addLayer(layer_substrate)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 11
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 3*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    # Disable material averaging (slicing not supported for soft particles)
    simulation.options().setUseAvgMaterials(False)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    from bornagain import ba_check
    ba_check.persistence_test(result)
