//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Refine/ParticlesInSlice.h
//! @brief     Defines function Refine::particlesInSlice.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif // SWIG

#ifndef BORNAGAIN_RESAMPLE_REFINE_PARTICLESINSLICE_H
#define BORNAGAIN_RESAMPLE_REFINE_PARTICLESINSLICE_H

#include <memory>
#include <vector>

class IParticle;
class Material;
class Slice;
class Span;
struct MesoOptions;

namespace Refine {

std::vector<std::unique_ptr<IParticle>> particlesInSlice(const IParticle* particle, const Span&,
                                                         const Slice* slice,
                                                         const Material& ambient_mat,
                                                         const MesoOptions& meso_options);

} // namespace Refine

#endif // BORNAGAIN_RESAMPLE_REFINE_PARTICLESINSLICE_H
