#!/usr/bin/env python3
"""
GISAS of sawtooth ripples on a 2D paracrystal lattice.

SawtoothRippleBox (100 x 20 x 10 nm, asymmetry d=-3 nm) rotated 30 deg
around Z, arranged on a BasicLattice2D (20 x 100 nm, 90 deg angle,
30 deg rotation) using Paracrystal2D with Gaussian pair-distance
distributions (sigma=150 x 600 nm, orientation 0 deg).
"""
import math
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    vacuum = ba.Vacuum()

    length_ripple = 100*nm
    ff = ba.SawtoothRippleBox(length_ripple, 20*nm, 10*nm, -3*nm)
    particle = ba.Particle(particle_mat, ff)
    particle.rotate(ba.RotationZ(math.pi/6))

    lattice = ba.BasicLattice2D(20*nm, length_ripple, math.pi/2, math.pi/6)
    structure = ba.Paracrystal2D(particle, lattice, 1000*nm, 1e4*nm, 1e4*nm)
    profile = ba.Profile2DGauss(150*nm, 600*nm, 0)
    structure.setProbabilityDistributions(profile, profile)

    vacuum_layer = ba.Layer(vacuum)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(structure)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1, 0.1*nm, 0.2*deg)
    n = <%= test_mode ? 25 : 100 %>
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    return ba.ScatteringSimulation(beam, sample, detector)


<%= include_partial("template/main/runsim_unitaspect.py",
    "ba.showSample3D(sample, sample_size=1000*nm, seed=0)") -%>
