//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/Cylinder.cpp
//! @brief     Implements class Cylinder.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/HardParticle/Cylinder.h"
#include "Base/Math/Bessel.h"
#include "Base/Math/Functions.h"
#include "Base/Util/Assert.h"
#include "Sample/Shape/DoubleEllipse.h"
#include <numbers>

using std::numbers::pi;

Cylinder::Cylinder(double radius, double height)
    : HardFormfactor()
    , m_radius(radius)
    , m_height(height)
{
    Cylinder::validate();
}

complex_t Cylinder::formfactorAtBottom(C3 q) const
{
    ASSERT(m_validated);
    const double R = m_radius;
    const double H = m_height;

    const complex_t qH2 = q.z() * H / 2.;
    const complex_t qR = std::sqrt(q.x() * q.x() + q.y() * q.y()) * R;

    const complex_t axial_part = H * Math::sinc(qH2);
    const complex_t radial_part = (2 * pi) * R * R * Math::Bessel::J1c(qR);

    return radial_part * axial_part * exp_I(qH2);
}

void Cylinder::validate() const
{
    if (m_radius <= 0)
        throw std::runtime_error("radius <= 0");
    if (m_height <= 0)
        throw std::runtime_error("height <= 0");

    // TODO improve!
    m_shape3D = std::make_unique<DoubleEllipseZ>(m_radius, m_radius, m_height, m_radius, m_radius);

    m_validated = true;
}

bool Cylinder::contains(const R3& position) const
{
    double R = radius();
    double H = height();

    if (std::abs(position.x()) > R || std::abs(position.y()) > R || position.z() < 0
        || position.z() > H)
        return false;

    if (std::pow(position.x() / R, 2) + std::pow(position.y() / R, 2) <= 1)
        return true;

    return false;
}
