#!/usr/bin/env python3
"""
GISAS of a centered square lattice (C2mm) with a two-cylinder basis.

The unit cell contains two cylinders at (0,0,0) and (5,-5,0) nm on a
20 nm × 20 nm square lattice, forming a centered (C2mm) arrangement.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3
import math


def get_sample():
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    vacuum = ba.Vacuum()

    ff = ba.Cylinder(5 * nm, 5 * nm)
    cylinder = ba.Particle(particle_mat, ff)

    basis = ba.Compound()
    basis.addComponents(cylinder, [R3(0, 0, 0), R3(5 * nm, -5 * nm, 0)])

    lattice = ba.BasicLattice2D(20 * nm, 20 * nm, 90 * deg, 0)
    structure = ba.Crystal2D(basis, lattice)
    profile = ba.Profile2DCauchy(300 / (2 * math.pi) * nm,
                                 100 / (2 * math.pi) * nm, 0)
    structure.setDecayFunction(profile)

    vacuum_layer = ba.Layer(vacuum)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(structure)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


<%= include_partial("template/sim/gisas_standard_wide.py") -%>


<%= include_partial("template/main/runsim_unitaspect.py",
    "ba.showSample3D(sample, sample_size=240*nm, seed=0)") -%>
