#!/usr/bin/env python3
"""
GISAS of two boxes rotated first around Z then around Y.

Two half-length boxes (25 × 20 × 10 nm) form a composition, rotated
sequentially 90° around Z then 90° around Y, and embedded in a 100 nm
Teflon layer. Material averaging is disabled.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3

layer_thickness = 100 * nm
length = 50 * nm
width = 20 * nm
height = 10 * nm


def get_sample():
    vacuum = ba.Vacuum()
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Ag", particle_color, 1.245e-5, 5.419e-7)
    teflon_color = (0.93, 0.72, 0.25)
    teflon_mat = ba.RefractiveMaterial("Teflon", teflon_color, 2.9e-6, 6.019e-9)
    substrate_color = (0.24, 0.50, 0.74)
    substrate_mat = ba.RefractiveMaterial(
        "Substrate2", substrate_color, 3.212e-6, 3.244e-8)

    box = ba.Particle(particle_mat, ba.Box(length / 2, width, height))
    composition = ba.Compound()
    composition.addComponent(box, R3())
    composition.addComponent(box, R3(length / 2, 0, 0))
    composition.rotate(ba.RotationZ(90 * deg))
    composition.rotate(ba.RotationY(90 * deg))
    composition.translate(R3(0, 0, -layer_thickness / 2))

    vacuum_layer = ba.Layer(vacuum)
    middle_layer = ba.Layer(teflon_mat, layer_thickness)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(ba.Dilute2D(5e-4, composition))

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(middle_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1 * nm, 0.2 * deg)
    n = 200
    detector = ba.SphericalDetector(n, -2 * deg, 2 * deg, n, 0, 2 * deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    simulation.options().setUseAvgMaterials(False)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    plotargs = bp.parse_commandline()
    bp.plot_datafield(result, unit_aspect=1, **plotargs)
    bp.export(**plotargs)
