#!/usr/bin/env python3
"""
GISAS from a periodic multilayer with self-affine fractal roughness.

Five repetitions of [PartA (2.5 nm) / PartB (5 nm)] on a substrate,
with correlated roughness described by a self-affine fractal model and
tanh transition profile.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    vacuum = ba.Vacuum()
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 15e-6, 0)
    part_a_color = (0.86, 0.24, 0.18)
    part_a_mat = ba.RefractiveMaterial("PartA", part_a_color, 5e-6, 0)
    part_b_color = (0.25, 0.65, 0.35)
    part_b_mat = ba.RefractiveMaterial("PartB", part_b_color, 10e-6, 0)

    autocorr = ba.SelfAffineFractalModel(1 * nm, 0.3, 5 * nm)
    transient = ba.TanhTransient()
    crosscorr = ba.CommonDepthCrosscorrelation(10 * nm)
    roughness = ba.Roughness(autocorr, transient, crosscorr)

    vacuum_layer = ba.Layer(vacuum)
    partA_layer = ba.Layer(part_a_mat, 2.5 * nm, roughness)
    partB_layer = ba.Layer(part_b_mat, 5 * nm, roughness)
    substrate_layer = ba.Layer(substrate_mat, roughness)

    stack = ba.LayerStack(5)
    stack.addLayer(partA_layer)
    stack.addLayer(partB_layer)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addStack(stack)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 11
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    from bornagain import ba_check
    ba_check.persistence_test(result)
