//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/Spheroid.h
//! @brief     Defines class Spheroid.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_SPHEROID_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_SPHEROID_H

#include "Sample/Particle/HardFormfactor.h"

//! A full spheroid (an ellipsoid with two equal axes, hence with circular cross section)

class Spheroid : public HardFormfactor {
public:
    Spheroid(double radius_xy, double radius_z);

    complex_t formfactorAtBottom(C3 q) const override;

    std::string className() const final { return "Spheroid"; }

#ifndef SWIG
    Spheroid* clone() const override { return new Spheroid(m_radius_xy, m_radius_z); }
    void validate() const override;

    double radiusXY() const { return m_radius_xy; }
    double radiusZ() const { return m_radius_z; }

    double radialExtension() const override { return m_radius_xy; }
    bool contains(const R3& position) const override;

private:
    double m_radius_xy;
    double m_radius_z;
#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_SPHEROID_H
