//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/SoftParticle/FuzzySphere.h
//! @brief     Defines and implements class FuzzySphere.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_SOFTPARTICLE_FUZZYSPHERE_H
#define BORNAGAIN_SAMPLE_SOFTPARTICLE_FUZZYSPHERE_H

#include "Sample/Particle/SoftFormfactor.h"
#include <memory>

//! A sphere with gaussian radius distribution.

class FuzzySphere : public SoftFormfactor {
public:
    FuzzySphere(double mean, double sigma);

    complex_t formfactorAtBottom(C3 q) const override;

    std::string className() const final { return "FuzzySphere"; }

#ifndef SWIG
    FuzzySphere* clone() const override { return new FuzzySphere(m_radius, m_sigma); }
    void validate() const override;

    double radialExtension() const override { return m_radius; }
    double fuzziness() const { return m_sigma; }

private:
    double m_radius;
    double m_sigma;
#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_SOFTPARTICLE_FUZZYSPHERE_H
