#!/usr/bin/env python3
"""
GISAS of a 3D array of spheres enclosed in a cylindrical envelope.

Spheres of radius 4 nm are placed on a 10 nm cubic grid within a
cylinder of radius (n_lim+0.5)·L and height (2·n_lim+1)·L. The
resulting clusters are deposited at low density (Dilute2D).
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-6, 2e-8)
    vacuum = ba.Vacuum()

    n_lim = 1
    L = 10 * nm
    R = 4 * nm

    ff = ba.Sphere(R)
    sphere = ba.Particle(particle_mat, ff)

    size = L * (2 * n_lim + 1 - 1) + 2 * R
    outer_r = size / 2

    positions = []
    for i in range(-n_lim, n_lim + 1):
        for j in range(-n_lim, n_lim + 1):
            for k in range(0, 2 * n_lim + 1):
                x, y, z = i * L, j * L, k * L
                if x**2 + y**2 <= outer_r**2 and 0 <= z <= size:
                    positions.append(R3(x, y, z))

    basis = ba.Compound()
    basis.addComponents(sphere, positions)

    vacuum_layer = ba.Layer(vacuum)
    substrate_layer = ba.Layer(substrate_mat)
    vacuum_layer.deposit2D(ba.Dilute2D(0.001, basis))

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


<%= include_partial("template/sim/gisas_standard_wide.py") -%>


<%= include_partial("template/main/runsim_unitaspect.py",
    "ba.showSample3D(sample, sample_size=250*nm, seed=0)") -%>
