#!/usr/bin/env python3
"""
Cylinder form factor in Born approximation, beam incident from below.

Cylinders (5 × 5 nm) in a vacuum matrix, with the beam angle set to
-0.2° (negative grazing angle), corresponding to a beam that enters
the sample from below the substrate. The detector is placed below the
horizon (alpha_f from -2° to 0°).
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)
    vacuum = ba.Vacuum()

    ff = ba.Cylinder(5*nm, 5*nm)
    particle = ba.Particle(particle_mat, ff)

    layer_top = ba.Layer(vacuum)
    layer_bottom = ba.Layer(vacuum)
    layer_top.deposit2D(ba.Dilute2D(0.001, particle))

    sample = ba.Sample()
    sample.addLayer(layer_top)
    sample.addLayer(layer_bottom)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, -0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, -2*deg, 0)
    return ba.ScatteringSimulation(beam, sample, detector)


if __name__ == '__main__':
    sample = get_sample()
    ba.showSample3D(sample, sample_size=120*nm, seed=0)
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_datafield(result, unit_aspect=1)
    bp.plt.show()
