//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/Disordered3D.h
//! @brief     Defines class Disordered3D.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_AGGREGATE_DISORDERED3D_H
#define BORNAGAIN_SAMPLE_AGGREGATE_DISORDERED3D_H

#include "Sample/Aggregate/Struct3D.h"

//! Disordered 3D particle assembly with a given volume number density.

class Disordered3D : public Struct3D {
public:
    ~Disordered3D() override;

    std::string className() const override { return "Disordered3D"; }

#ifndef SWIG
    Disordered3D* clone() const override;

    double volumeNumberDensity() const override { return m_volume_density; }

protected:
    Disordered3D(double volume_density, const IParticle& particle);

private:
    Disordered3D(double coverage, double volume_density, const IParticle& particle);

    double m_volume_density;

#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_AGGREGATE_DISORDERED3D_H
