#!/usr/bin/env python3
"""
Dilute film of small spheres
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2, nm3

particle_radius = 4*nm
particle_height = 2*particle_radius
area_density = .016/nm2


def get_sample(thickness):
    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 1.5e-05, 2e-08)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 1e-06, 2e-08)
    vacuum = ba.Vacuum()

    # Particles
    ff = ba.Sphere(particle_radius)
    particle = ba.Particle(particle_mat, ff)

    # Layers
    layer_1 = ba.Layer(vacuum)
    layer_3 = ba.Layer(substrate_mat)
    if thickness > 0:
        layer_2 = ba.Layer(substrate_mat, thickness)
        volume_density = area_density/thickness
        layer_2.fill3D(ba.Dilute3D(volume_density, particle))
    else:
        layer_1.deposit2D(ba.Dilute2D(area_density, particle))

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_1)
    if thickness > 0:
        sample.addLayer(layer_2)
    sample.addLayer(layer_3)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 11
    # Just compute a 1d cut at phi=0
    detector = ba.SphericalDetector(1, -1*deg, 1*deg, n, 0., 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    thicknesses = [
        0*nm, particle_height, 2*particle_height, 4*particle_height,
        8*particle_height
    ]
    labels = ["Monolayer"]
    labels.extend(f'{t/nm:.0f} nm' for t in thicknesses[1:])
    samples = [ get_sample(t) for t in thicknesses ]
    results = [ get_simulation(sample).simulate() for sample in samples ]
    from bornagain import ba_check
    ba_check.persistence_test(results)
