//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Element/Element.h
//! @brief     Defines class Element.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif // SWIG
#ifndef BORNAGAIN_SAMPLE_ELEMENT_IELEMENT_H
#define BORNAGAIN_SAMPLE_ELEMENT_IELEMENT_H

#include "Base/Spin/SpinMatrix.h"

class Element {
public:
    Element() = default;

    Element(const SpinMatrix& polarizer, const SpinMatrix& analyzer)
        : m_polarizer(polarizer)
        , m_analyzer(analyzer)
    {
    }

    const SpinMatrix& polarizer() const { return m_polarizer; }
    const SpinMatrix& analyzer() const { return m_analyzer; }

protected:
    const SpinMatrix m_polarizer{SpinMatrix::One() / 2.0};
    const SpinMatrix m_analyzer{SpinMatrix::One()};
};

#endif // BORNAGAIN_SAMPLE_ELEMENT_IELEMENT_H
