#!/usr/bin/env python3
"""
Soft spheres (FuzzySphere) in a finite layer.

Soft particles have diffuse boundaries modelled by a Debye-Waller factor.
Slicing is not supported for soft particles; material averaging is disabled.

The layer thickness is 8*R. The particle center is shifted up by 4*R (= thickness/2)
so that almost all of the particle sits inside the finite layer.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2


def get_sample():
    R = 5 * nm
    thickness = 8 * R  # 40 nm

    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-04, 2e-08)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-06, 2e-08)
    vacuum = ba.Vacuum()

    # Soft particle (FuzzySphere with Gaussian fuzziness)
    ff = ba.FuzzySphere(R, 0.5 * nm)
    particle = ba.Particle(particle_mat, ff)
    # Shift center to thickness/2 (deposit2D anchors the bottom, so compensate)
    particle.translate(0, 0, thickness / 2 - particle.height_below_center())

    # Layers: vacuum / finite film / substrate
    layer_vacuum = ba.Layer(vacuum)
    layer_film = ba.Layer(vacuum, thickness)
    layer_film.deposit2D(ba.Dilute2D(1e-3 / nm2, particle))
    layer_substrate = ba.Layer(substrate_mat)

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_vacuum)
    sample.addLayer(layer_film)
    sample.addLayer(layer_substrate)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1 * nm, 0.2 * deg)
    n = 200
    detector = ba.SphericalDetector(n, -2 * deg, 2 * deg, n, 0, 3 * deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    # Disable material averaging (slicing not supported for soft particles)
    simulation.options().setUseAvgMaterials(False)
    return simulation

if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    plotargs = bp.parse_commandline()
    bp.plot_datafield(result, unit_aspect=1, **plotargs)
    bp.export(**plotargs)
