//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Particle/SoftFormfactor.h
//! @brief     Defines class SoftFormfactor.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2026
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_PARTICLE_SOFTFORMFACTOR_H
#define BORNAGAIN_SAMPLE_PARTICLE_SOFTFORMFACTOR_H

#include "Sample/Particle/Formfactor.h"

//! Abstract base class for soft particle form factors with diffuse boundaries.
//!
//! Soft particles have no well-defined boundary, so geometric containment tests
//! are not meaningful.

class SoftFormfactor : public Formfactor {
#ifndef SWIG
public:
    SoftFormfactor* clone() const override = 0;
#endif // SWIG
};

#endif // BORNAGAIN_SAMPLE_PARTICLE_SOFTFORMFACTOR_H
