#!/usr/bin/env python3
"""
Dense film of small spheres: scattering intensity for different film thicknesses.
Uses the Percus-Yevick approximation.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm3


def get_sample(thickness):
    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 1e-5, 0)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 1e-6, 0)
    vacuum = ba.Vacuum()

    # Particles
    ff = ba.Sphere(4*nm)
    particle = ba.Particle(particle_mat, ff)

    # Layers
    layer_1 = ba.Layer(vacuum)
    layer_2 = ba.Layer(vacuum, thickness)
    layer_3 = ba.Layer(substrate_mat)
    layer_2.fill3D(ba.Dilute3D(1e-8, particle))

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)
    sample.addLayer(layer_3)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 11
    # Just compute a 1d cut at phi=0
    detector = ba.SphericalDetector(1, -1*deg, 1*deg, n, 0., 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    thicknesses = [10, 31.6, 100, 316, 1000]
    samples = [get_sample(t) for t in thicknesses]
    results = [get_simulation(sample).simulate() for sample in samples]
    from bornagain import ba_check
    ba_check.persistence_test(results)
