#!/usr/bin/env python3
"""
Cylindrical particle made from two materials.
Particle crosses air/substrate interface.
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2, R3


def get_sample():
    """
    A multilayer with substrate/air layers.
    Vacuum layer contains cylindrical particles made of two materials.
    Particle shifted down to cross interface.
    """

    # Materials
    ag_color = (0.86, 0.24, 0.18)
    ag_mat = ba.RefractiveMaterial("Ag", ag_color, 1.245e-05, 5.419e-07)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 3.212e-06,
                                               3.244e-08)
    teflon_color = (0.93, 0.72, 0.25)
    teflon_mat = ba.RefractiveMaterial("Teflon", teflon_color, 2.9e-06, 6.019e-09)
    vacuum = ba.Vacuum()

    # Form factors
    ff_1 = ba.Cylinder(10*nm, 4*nm)
    ff_2 = ba.Cylinder(10*nm, 10*nm)

    # Particles
    subparticle_1 = ba.Particle(ag_mat, ff_1)
    subparticle_1.translate(R3(0, 0, 10*nm))
    subparticle_2 = ba.Particle(teflon_mat, ff_2)

    # Composition of particles at specific positions
    particle = ba.Compound()
    particle.addComponent(subparticle_1)
    particle.addComponent(subparticle_2)
    particle.translate(R3(0, 0, -10*nm))

    # Layers
    layer_1 = ba.Layer(vacuum)
    layer_1.deposit2D(ba.Dilute2D(0.0001/nm2, particle))
    layer_2 = ba.Layer(substrate_mat)

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 100
    detector = ba.SphericalDetector(n, -1*deg, 1*deg, n, 0., 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    plotargs = bp.parse_commandline()
    bp.plot_datafield(result, unit_aspect=1, **plotargs)
    bp.export(**plotargs)
